@@ -1,10 +1,11 @@
 /*
- *	$Id: ccheader,v 1.2 2001/12/01 23:37:26 ken Exp $
+ *	$Id$
  */
 
 #include  <vte/vte.h>
 #include  <vte/reaper.h>
 
+#include  <pwd.h>			/* getpwuid */
 #include  <X11/keysym.h>
 #include  <gdk/gdkx.h>
 #include  <gtk/gtksignal.h>
@@ -19,7 +20,6 @@
 #include  <kiklib/kik_locale.h>
 #include  <kiklib/kik_conf_io.h>
 #include  <kiklib/kik_pty.h>		/* kik_pty_helper_set_flag */
-#include  <mkf/mkf_utf8_conv.h>
 #include  <ml_str_parser.h>
 #include  <ml_term_manager.h>
 #include  <x_screen.h>
@@ -38,6 +38,10 @@
 #define  __DEBUG
 #endif
 
+#if  0
+#define  ATTACH_STYLE
+#endif
+
 #ifndef  I_
 #define  I_(a)  a
 #endif
@@ -60,15 +64,6 @@
 
 #define  STATIC_PARAMS (G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)
 
-/*
- * XXX
- * char length is max 8 bytes.
- * I think this is enough , but I'm not sure.
- * This macro used for UTF8 and UTF16.
- * (Same as x_screen.c)
- */
-#define  UTF_MAX_CHAR_SIZE  (8 * (MAX_COMB_SIZE + 1))
-
 
 struct _VteTerminalPrivate
 {
@@ -101,6 +96,11 @@ struct _VteTerminalPrivate
 	u_int  pix_width ;
 	u_int  pix_height ;
 	x_picture_modifier_t *  pic_mod ;
+
+/* GRegex was not supported */
+#if  (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 14)
+	GRegex *  regex ;
+#endif
 } ;
 
 enum
@@ -163,9 +163,122 @@ static char *  false = "false" ;
 
 /* --- static functions --- */
 
+static int
+selection(
+	x_selection_t *  sel ,
+	int  char_index_1 ,
+	int  row_1 ,
+	int  char_index_2 ,
+	int  row_2
+	)
+{
+	x_sel_clear( sel) ;
+
+	x_start_selection( sel , char_index_1 - 1 , row_1 , char_index_1 , row_1) ;
+	x_selecting( sel , char_index_2 , row_2) ;
+	x_stop_selecting( sel) ;
+
+	return  1 ;
+}
+
+/* GRegex was not supported */
+#if  (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 14)
+static int
+match(
+	size_t *  beg ,
+	size_t *  len ,
+	void *  regex ,
+	u_char *  str ,
+	int  backward
+	)
+{
+	GMatchInfo *  info ;
+
+	if( g_regex_match( regex , str , 0 , &info))
+	{
+		gchar *  word ;
+		u_char *  p ;
+
+		p = str ;
+
+		do
+		{
+			word = g_match_info_fetch( info , 0) ;
+
+			p = strstr( p , word) ;
+			*beg = p - str ;
+			*len = strlen( word) ;
+
+			g_free( word) ;
+
+			p += (*len) ;
+		}
+		while( g_match_info_next( info , NULL)) ;
+
+		g_match_info_free( info) ;
+
+		return  1 ;
+	}
+	
+	return  0 ;
+}
+
+static gboolean
+search_find(
+	VteTerminal *  terminal ,
+	int  backward
+	)
+{
+	int  beg_char_index ;
+	int  beg_row ;
+	int  end_char_index ;
+	int  end_row ;
+
+	if( ! GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
+	{
+		return  FALSE ;
+	}
+
+	if( ml_term_search_find( terminal->pvt->term , &beg_char_index , &beg_row ,
+			&end_char_index , &end_row , terminal->pvt->regex , backward))
+	{
+		gdouble  value ;
+
+		selection( &terminal->pvt->screen->sel ,
+			beg_char_index , beg_row , end_char_index , end_row) ;
+
+		value = ml_term_get_num_of_logged_lines( terminal->pvt->term) +
+				(beg_row >= 0 ? 0 : beg_row) ;
+
+	#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
+		gtk_adjustment_set_value( terminal->adjustment , value) ;
+	#else
+		VTE_WIDGET(screen)->adjustment->value = value ;
+		gtk_adjustment_value_changed( terminal->adjustment) ;
+	#endif
+
+	#if  1
+		/*
+		 * XXX
+		 * Dirty hack, but without this, selection() above is not reflected to window
+		 * if aother word is hit in the same line. (If row is not changed,
+		 * gtk_adjustment_set_value() doesn't call x_screen_scroll_to().)
+		 */
+		x_window_update( &terminal->pvt->screen->window , 1 /* UPDATE_SCREEN */) ;
+	#endif
+
+		return  TRUE ;
+	}
+	else
+	{
+		return  FALSE ;
+	}
+}
+#endif
+
 #if  (GTK_MAJOR_VERSION == 1) || ((GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 12))
 /* gdk_color_to_string() was not supported by gtk+ < 2.12. */
-gchar *
+static gchar *
 gdk_color_to_string(
 	const GdkColor *  color
 	)
@@ -218,7 +331,7 @@ vte_terminal_io(
 	gpointer  data		/* ml_term_t */
 	)
 {
-	ml_term_parse_vt100_sequence( data) ;
+	ml_term_parse_vt100_sequence( (ml_term_t*)data) ;
 	
 	ml_close_dead_terms() ;
 	
@@ -458,19 +571,13 @@ line_scrolled_out(
 	)
 {
 	x_screen_t *  screen ;
-	int  upper ;
-	int  value ;
+	gdouble  upper ;
+	gdouble  value ;
 	
 	screen = p ;
 
 	VTE_WIDGET(screen)->pvt->line_scrolled_out( p) ;
 
-	if( ( upper = gtk_adjustment_get_upper( VTE_WIDGET(screen)->adjustment))
-		== ml_term_get_log_size( VTE_WIDGET(screen)->pvt->term))
-	{
-		return ;
-	}
-
 	/*
 	 * line_scrolled_out is called in vt100 mode
 	 * (after ml_xterm_event_listener_t::start_vt100 event), so
@@ -480,26 +587,42 @@ line_scrolled_out(
 	VTE_WIDGET(screen)->pvt->adj_value_changed_by_myself = 1 ;
 
 	value = gtk_adjustment_get_value( VTE_WIDGET(screen)->adjustment) ;
-	
-#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
-	gtk_adjustment_set_upper( VTE_WIDGET(screen)->adjustment , upper + 1) ;
-	if( ml_term_is_backscrolling( VTE_WIDGET(screen)->pvt->term) != BSM_STATIC)
+
+	if( ( upper = gtk_adjustment_get_upper( VTE_WIDGET(screen)->adjustment))
+	    < ml_term_get_log_size( VTE_WIDGET(screen)->pvt->term) + VTE_WIDGET(screen)->row_count)
 	{
-		gtk_adjustment_set_value( VTE_WIDGET(screen)->adjustment , value + 1) ;
+	#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
+		gtk_adjustment_set_upper( VTE_WIDGET(screen)->adjustment , upper + 1) ;
+	#else
+		VTE_WIDGET(screen)->adjustment->upper ++ ;
+		gtk_adjustment_changed( VTE_WIDGET(screen)->adjustment) ;
+	#endif
+
+		if( ml_term_is_backscrolling( VTE_WIDGET(screen)->pvt->term) != BSM_STATIC)
+		{
+		#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
+			gtk_adjustment_set_value( VTE_WIDGET(screen)->adjustment , value + 1) ;
+		#else
+			VTE_WIDGET(screen)->adjustment->value ++ ;
+			gtk_adjustment_value_changed( VTE_WIDGET(screen)->adjustment) ;
+		#endif
+		}
 	}
-#else
-	VTE_WIDGET(screen)->adjustment->upper ++ ;
-	gtk_adjustment_changed( VTE_WIDGET(screen)->adjustment) ;
-	if( ml_term_is_backscrolling( VTE_WIDGET(screen)->pvt->term) != BSM_STATIC)
+	else if( ml_term_is_backscrolling( VTE_WIDGET(screen)->pvt->term) == BSM_STATIC &&
+			value > 0)
 	{
-		VTE_WIDGET(screen)->adjustment->value ++ ;
+	#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
+		gtk_adjustment_set_value( VTE_WIDGET(screen)->adjustment , value - 1) ;
+	#else
+		VTE_WIDGET(screen)->adjustment->value -- ;
 		gtk_adjustment_value_changed( VTE_WIDGET(screen)->adjustment) ;
+	#endif
 	}
-#endif
 
 #ifdef  __DEBUG
-	kik_debug_printf( KIK_DEBUG_TAG " line_scrolled_out upper %d value %d\n" ,
-		upper + 1 , value + 1) ;
+	kik_debug_printf( KIK_DEBUG_TAG " line_scrolled_out upper %f value %f\n" ,
+		gtk_adjustment_get_upper( VTE_WIDGET(screen)->adjustment) ,
+		gtk_adjustment_get_value( VTE_WIDGET(screen)->adjustment)) ;
 #endif
 }
 
@@ -824,8 +947,8 @@ vte_terminal_filter(
 		return  GDK_FILTER_REMOVE ;
 	}
 
-	if( ( ((XEvent*)xevent)->xany.type == KeyPress ||
-		((XEvent*)xevent)->xany.type == KeyRelease) )
+	if( ( ((XEvent*)xevent)->type == KeyPress ||
+		((XEvent*)xevent)->type == KeyRelease) )
 	{
 		is_key_event = 1 ;
 	}
@@ -853,13 +976,18 @@ vte_terminal_filter(
 			 * This processing is added for key binding of popup menu.
 			 */
 			if( is_key_event &&
-			    ((XEvent*)xevent)->xany.window == disp.roots[count]->my_window &&
-			    ! disp.roots[count]->is_focused)
+			    ((XEvent*)xevent)->xany.window == disp.roots[count]->my_window)
 			{
-				((XEvent*)xevent)->xany.window =
-					gdk_x11_drawable_get_xid( GTK_WIDGET(terminal)->window) ;
+				ml_term_search_reset_position( terminal->pvt->term) ;
 
-				return  GDK_FILTER_CONTINUE ;
+				if( ! disp.roots[count]->is_focused)
+				{
+					((XEvent*)xevent)->xany.window =
+						gdk_x11_drawable_get_xid(
+							GTK_WIDGET(terminal)->window) ;
+
+					return  GDK_FILTER_CONTINUE ;
+				}
 			}
 		}
 		else
@@ -1135,11 +1263,42 @@ vte_terminal_realize(
 				GDK_WA_X | GDK_WA_Y |
 				(attr.visual ? GDK_WA_VISUAL : 0) |
 				(attr.colormap ? GDK_WA_COLORMAP : 0) ) ;
+
+	/*
+	 * Note that hook key and button events in vte_terminal_filter doesn't work without this.
+	 */
 	gdk_window_set_user_data( widget->window , widget) ;
 
 	GTK_WIDGET_SET_FLAGS( widget , GTK_REALIZED) ;
 
+#if  1
+	if( widget->style->font_desc)
+	{
+		pango_font_description_free( widget->style->font_desc) ;
+		widget->style->font_desc = NULL ;
+	}
+#endif
+
+#ifdef  ATTACH_STYLE
 	widget->style = gtk_style_attach( widget->style , widget->window) ;
+#endif
+
+	/*
+	 * private_font(_desc) should be NULL if widget->style->font_desc is set NULL above.
+	 */
+#if  0
+	if( widget->style->private_font)
+	{
+		gdk_font_unref( widget->style->private_font) ;
+		widget->style->private_font = NULL ;
+	}
+
+	if( widget->style->private_font_desc)
+	{
+		pango_font_description_free( widget->style->private_font_desc) ;
+		widget->style->private_font_desc = NULL ;
+	}
+#endif
 
 	if( ! timeout_registered)
 	{
@@ -1239,7 +1398,9 @@ vte_terminal_unrealize(
 		gtk_widget_unmap( widget) ;
 	}
 
+#ifdef  ATTACH_STYLE
 	gtk_style_detach( widget->style) ;
+#endif
 	gdk_window_destroy( widget->window) ;
 	widget->window = NULL ;
 
@@ -1403,7 +1564,7 @@ vte_terminal_class_init(
 	x_shortcut_init( &shortcut) ;
 	x_termcap_init( &termcap) ;
 	x_xim_init( 1) ;
-	x_font_use_point_size_for_xft( 1) ;
+	x_font_use_point_size_for_fc( 1) ;
 	x_set_button3_behavior( "none") ;
 	ml_term_manager_enable_zombie_pty( 1) ;
 
@@ -1412,7 +1573,7 @@ vte_terminal_class_init(
 	x_prepare_for_main_config( conf) ;
 	x_main_config_init( &main_config , conf , 1 , argv) ;
 
-#ifdef  USE_TYPE_XFT
+#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( main_config.type_engine == TYPE_XCORE)
 	{
 		/*
@@ -1425,7 +1586,11 @@ vte_terminal_class_init(
 		if( ( value = kik_conf_get_value( conf , "type_engine")) == NULL ||
 			strcmp( value , "xcore") != 0)
 		{
+		#ifdef  USE_TYPE_XFT
 			main_config.type_engine = TYPE_XFT ;
+		#else
+			main_config.type_engine = TYPE_CAIRO ;
+		#endif
 		}
 	}
 #endif
@@ -1765,16 +1930,16 @@ vte_terminal_init(
 		G_CALLBACK(vte_terminal_hierarchy_changed) , NULL) ;
 
 	terminal->pvt->term =
-		ml_create_term( 80 /* main_config.cols */ , 25 /* main_config.rows */ ,
+		ml_create_term( main_config.cols , main_config.rows ,
 			main_config.tab_size , main_config.num_of_log_lines ,
 			main_config.encoding , main_config.is_auto_encoding , 
 			main_config.unicode_policy , main_config.col_size_of_width_a ,
 			main_config.use_char_combining , main_config.use_multi_col_char ,
-			main_config.use_bidi , main_config.bidi_mode , 
+			main_config.use_bidi , main_config.bidi_mode , main_config.use_ind ,
 			x_termcap_get_bool_field(
 				x_termcap_get_entry( &termcap , main_config.term_type) , ML_BCE) ,
 			main_config.use_dynamic_comb , main_config.bs_mode ,
-			main_config.vertical_mode , main_config.iscii_lang_type) ;
+			main_config.vertical_mode) ;
 
 	if( main_config.unicode_policy & NOT_USE_UNICODE_FONT ||
 		main_config.iso88591_font_for_usascii)
@@ -1794,7 +1959,7 @@ vte_terminal_init(
 		usascii_font_cs_changable = 1 ;
 	}
 
-	/* related to x_font_use_point_size_for_xft(1) in vte_terminal_class_init. */
+	/* related to x_font_use_point_size_for_fc(1) in vte_terminal_class_init. */
 	if( ( dpi = gdk_screen_get_resolution(
 		gtk_widget_get_screen( GTK_WIDGET(terminal)))) != -1)
 	{
@@ -1802,7 +1967,7 @@ vte_terminal_init(
 		kik_debug_printf( KIK_DEBUG_TAG " Setting dpi %f\n" , dpi) ;
 	#endif
 
-		x_font_set_dpi_for_xft( dpi) ;
+		x_font_set_dpi_for_fc( dpi) ;
 	}
 
 	font_man = x_font_manager_new( disp.display ,
@@ -1880,6 +2045,11 @@ vte_terminal_init(
 	terminal->pvt->audible_bell = (main_config.bel_mode == BEL_SOUND) ;
 	terminal->pvt->visible_bell = (main_config.bel_mode == BEL_VISUAL) ;
 
+/* GRegex was not supported */
+#if  (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 14)
+	terminal->pvt->regex = NULL ;
+#endif
+
 	terminal->window_title = ml_term_window_name( terminal->pvt->term) ;
 	terminal->icon_title = ml_term_icon_name( terminal->pvt->term) ;
 
@@ -1952,8 +2122,8 @@ vte_terminal_new()
 pid_t
 vte_terminal_fork_command(
 	VteTerminal *  terminal ,
-	const char *  command ,
-	char **  argv ,
+	const char *  command ,		/* If NULL, open default shell. */
+	char **  argv ,			/* If NULL, open default shell. */
 	char **  envv ,
 	const char *  directory ,
 	gboolean  lastlog ,
@@ -1963,12 +2133,33 @@ vte_terminal_fork_command(
 {
 	if( ! terminal->pvt->term->pty)
 	{
-	#ifdef  __DEBUG
+	#ifdef  DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " forking with %s\n" , command) ;
 	#endif
 	
+		if( ! command)
+		{
+			if( ! ( command = getenv( "SHELL")) || *command == '\0')
+			{
+				struct passwd *  pw ;
+
+				if( ( pw = getpwuid( getuid())) == NULL ||
+				    *( command = pw->pw_shell) == '\0')
+				{
+					command = "/bin/sh" ;
+				}
+			}
+		}
+
+		if( ! argv)
+		{
+			argv = alloca( sizeof(char*) * 2) ;
+			argv[0] = command ;
+			argv[1] = NULL ;
+		}
+
 		kik_pty_helper_set_flag( lastlog , utmp , wtmp) ;
-		
+
 		if( ! ml_term_open_pty( terminal->pvt->term , command , argv , envv ,
 			gdk_display_get_name( gtk_widget_get_display( GTK_WIDGET(terminal))) ,
 			NULL , NULL , NULL) )
@@ -2043,7 +2234,7 @@ vte_terminal_forkpty(
 {
 	if( ! terminal->pvt->term->pty)
 	{
-	#ifdef  __DEBUG
+	#ifdef  DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " forking pty\n") ;
 	#endif
 	
@@ -2130,7 +2321,7 @@ vte_terminal_copy_clipboard(
 		return ;
 	}
 
-	len = terminal->pvt->screen->sel.sel_len * UTF_MAX_CHAR_SIZE ;
+	len = terminal->pvt->screen->sel.sel_len * MLCHAR_UTF_MAX_SIZE ;
 	if( ! ( buf = alloca( len)))
 	{
 		return ;
@@ -2175,8 +2366,8 @@ vte_terminal_select_all(
 	VteTerminal *  terminal
 	)
 {
-	x_selection_t *  sel ;
-	u_int  row ;
+	int  beg_row ;
+	int  end_row ;
 	ml_line_t *  line ;
 	
 	if( ! GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
@@ -2184,30 +2375,19 @@ vte_terminal_select_all(
 		return ;
 	}
 	
-	sel = &terminal->pvt->screen->sel ;
-	
-	x_sel_clear( sel) ;
+	beg_row = - ml_term_get_num_of_logged_lines( terminal->pvt->term) ;
 
-	row = - ml_term_get_num_of_logged_lines( terminal->pvt->term) ;
-	x_start_selection( sel , -1 , row , 0 , row) ;
-
-	for( row = ml_term_get_rows( terminal->pvt->term) - 1 ; row > 0 ; row --)
+	for( end_row = ml_term_get_rows( terminal->pvt->term) - 1 ; end_row >= 0 ; end_row --)
 	{
-		line = ml_term_get_line( terminal->pvt->term , row) ;
-		if( ! ml_line_is_empty( line))
+		if( (line = ml_term_get_line( terminal->pvt->term , end_row)) &&
+		    ! ml_line_is_empty( line))
 		{
 			break ;
 		}
 	}
 
-	if( row == 0)
-	{
-		line = ml_term_get_line( terminal->pvt->term , 0) ;
-	}
-	
-	x_selecting( sel , ml_line_get_num_of_filled_cols( line) - 1 , row) ;
-
-	x_stop_selecting( sel) ;
+	selection( &terminal->pvt->screen->sel , 0 , beg_row ,
+			line->num_of_filled_chars - 1 , end_row) ;
 }
 
 void
@@ -2741,10 +2921,15 @@ vte_terminal_set_font_from_string(
 	const char *  name
 	)
 {
-#ifdef  __DEBUG
+#ifdef  DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " set_font_from_string %s\n" , name) ;
 #endif
 
+	if( ! name)
+	{
+		name = "monospace" ;
+	}
+
 	if( x_customize_font_file( "aafont" , "DEFAULT" , name , 0))
 	{
 		x_font_cache_unload_all() ;
@@ -2839,6 +3024,10 @@ vte_terminal_set_backspace_binding(
 	x_termcap_entry_t *  entry ;
 	char *  seq ;
 
+#ifdef  DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " set backtrace binding => %d\n") ;
+#endif
+
 	if( binding == VTE_ERASE_ASCII_BACKSPACE)
 	{
 		seq = "\x08" ;
@@ -3062,31 +3251,25 @@ vte_terminal_match_check(
 {
 	u_char *  buf ;
 	size_t  len ;
-	mkf_conv_t *  conv ;
 
 	if( ! vte_terminal_get_has_selection( terminal))
 	{
 		return  NULL ;
 	}
 
-	len = terminal->pvt->screen->sel.sel_len * UTF_MAX_CHAR_SIZE + 1 ;
+	len = terminal->pvt->screen->sel.sel_len * MLCHAR_UTF_MAX_SIZE + 1 ;
 	if( ! ( buf = g_malloc( len)))
 	{
 		return  NULL ;
 	}
-	
+
 	(*terminal->pvt->screen->ml_str_parser->init)( terminal->pvt->screen->ml_str_parser) ;
 	ml_str_parser_set_str( terminal->pvt->screen->ml_str_parser ,
 		terminal->pvt->screen->sel.sel_str , terminal->pvt->screen->sel.sel_len) ;
 
-	conv = mkf_utf8_conv_new() ;
-	(*conv->init)( conv) ;
-
-	len = (*conv->convert)( conv , buf , len , terminal->pvt->screen->ml_str_parser) ;
-
-	buf[len] = '\0' ;
-
-	(*conv->delete)( conv) ;
+	(*terminal->pvt->screen->utf_conv->init)( terminal->pvt->screen->utf_conv) ;
+	*(buf + (*terminal->pvt->screen->utf_conv->convert)( terminal->pvt->screen->utf_conv ,
+			buf , len , terminal->pvt->screen->ml_str_parser)) = '\0' ;
 
 	/* XXX */
 	*tag = 1 ;	/* For pattern including "http" (see vte_terminal_match_add_gregex) */
@@ -3102,6 +3285,19 @@ vte_terminal_search_set_gregex(
 	GRegex *  regex
 	)
 {
+	if( regex)
+	{
+		if( ! terminal->pvt->regex)
+		{
+			ml_term_search_init( terminal->pvt->term , match) ;
+		}
+	}
+	else
+	{
+		ml_term_search_final( terminal->pvt->term) ;
+	}
+	
+	terminal->pvt->regex = regex ;
 }
 
 GRegex *
@@ -3109,7 +3305,23 @@ vte_terminal_search_get_gregex(
 	VteTerminal *  terminal
 	)
 {
-	return  NULL ;
+	return  terminal->pvt->regex ;
+}
+
+gboolean
+vte_terminal_search_find_previous(
+	VteTerminal *  terminal
+	)
+{
+	return  search_find( terminal , 1) ;
+}
+
+gboolean
+vte_terminal_search_find_next(
+	VteTerminal *  terminal
+	)
+{
+	return  search_find( terminal , 0) ;
 }
 #endif
 
@@ -3129,22 +3341,6 @@ vte_terminal_search_get_wrap_around(
 	return  FALSE ;
 }
 
-gboolean
-vte_terminal_search_find_previous(
-	VteTerminal *  terminal
-	)
-{
-	return  FALSE ;
-}
-
-gboolean
-vte_terminal_search_find_next(
-	VteTerminal *  terminal
-	)
-{
-	return  FALSE ;
-}
-
 void
 vte_terminal_set_emulation(
 	VteTerminal *  terminal ,
