@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_dlfcn_dl.c,v 1.5 2005/02/18 14:32:00 seiichi Exp $
+ *	$Id$
  */
 
 #include  "kik_dlfcn.h"
@@ -15,8 +15,8 @@
 
 kik_dl_handle_t
 kik_dl_open(
-	char *  dirpath ,
-	char *  name
+	const char *  dirpath ,
+	const char *  name
 	)
 {
 	char *  path ;
@@ -69,7 +69,7 @@ kik_dl_close(
 void *
 kik_dl_func_symbol(
 	kik_dl_handle_t  handle ,
-	char *  symbol
+	const char *  symbol
 	)
 {
 	return  dlsym( handle , symbol) ;
@@ -77,7 +77,7 @@ kik_dl_func_symbol(
 
 int
 kik_dl_is_module(
-	char * name
+	const char * name
 	)
 {
 	size_t  len ;
