@@ -32,15 +32,21 @@ typedef struct  ml_bidi_state
 } ml_bidi_state_t ;
 
 
+#ifdef  USE_FRIBIDI
+
 ml_bidi_state_t *  ml_bidi_new(void) ;
 
+int  ml_bidi_copy( ml_bidi_state_t *  dst , ml_bidi_state_t *  src) ;
+
 int  ml_bidi_delete( ml_bidi_state_t *  state) ;
 
 int  ml_bidi_reset( ml_bidi_state_t *  state) ;
 
 int  ml_bidi( ml_bidi_state_t *  state , ml_char_t *  src , u_int  size , ml_bidi_mode_t  mode) ;
 
-ml_bidi_mode_t  ml_get_bidi_mode( char *  name) ;
+#endif	/* USE_FRIBIDI */
+
+ml_bidi_mode_t  ml_get_bidi_mode( const char *  name) ;
 
 char *  ml_get_bidi_mode_name( ml_bidi_mode_t  mode) ;
 
