@@ -31,29 +31,30 @@ typedef struct  ml_edit
 	ml_model_t  model ;
 	ml_cursor_t  cursor ;
 
-	int8_t  is_logging ;
-
-	u_int8_t *  tab_stops ;
 	u_int  tab_size ;
+	u_int8_t *  tab_stops ;
 
 	ml_char_t  bce_ch ;
 
 	/* used for line overlapping */
 	ml_line_t *  wraparound_ready_line ;
-	
+
 	int  scroll_region_beg ;
 	int  scroll_region_end ;
 
 	ml_edit_scroll_event_listener_t *  scroll_listener ;
-	
+
+	int8_t  is_logging ;
 	int8_t  is_relative_origin ;
 	int8_t  is_auto_wrap ;
+	int8_t  use_bce ;
 
 } ml_edit_t ;
 
 
 int  ml_edit_init( ml_edit_t *  edit , ml_edit_scroll_event_listener_t *  scroll_listener ,
-	u_int  num_of_cols , u_int  num_of_rows , u_int  tab_size , int  is_logging) ;
+	u_int  num_of_cols , u_int  num_of_rows , u_int  tab_size , int  is_logging ,
+	int  use_bce) ;
 
 int  ml_edit_final( ml_edit_t *  edit) ;
 
@@ -67,32 +68,20 @@ int  ml_edit_overwrite_chars( ml_edit_t *  edit , ml_char_t *  chars , u_int  nu
 
 int  ml_edit_delete_cols( ml_edit_t *  edit , u_int  delete_cols) ;
 
-int  ml_edit_delete_cols_bce( ml_edit_t *  edit , u_int  delete_cols) ;
-
 int  ml_edit_clear_cols( ml_edit_t *  edit , u_int  cols) ;
 
-int  ml_edit_clear_cols_bce( ml_edit_t *  edit , u_int  cols) ;
-
 int  ml_edit_insert_new_line( ml_edit_t *  edit) ;
 
 int  ml_edit_delete_line( ml_edit_t *  edit) ;
 
 int  ml_edit_clear_line_to_right( ml_edit_t *  edit) ;
 
-int  ml_edit_clear_line_to_right_bce( ml_edit_t *  edit) ;
-
 int  ml_edit_clear_line_to_left( ml_edit_t *  edit) ;
 
-int  ml_edit_clear_line_to_left_bce( ml_edit_t *  edit) ;
-
 int  ml_edit_clear_below( ml_edit_t *  edit) ;
 
-int  ml_edit_clear_below_bce( ml_edit_t *  edit) ;
-
 int  ml_edit_clear_above( ml_edit_t *  edit) ;
 
-int  ml_edit_clear_above_bce( ml_edit_t *  edit) ;
-
 int  ml_edit_fill_all( ml_edit_t *  edit , ml_char_t *  ch) ;
 	
 int  ml_edit_set_scroll_region( ml_edit_t *  edit , int  beg , int  end) ;
