@@ -10,6 +10,9 @@
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_util.h>	/* K_MIN */
 
+#include  "ml_char_encoding.h"
+#include  "ml_str_parser.h"
+
 
 #define  ROW_IN_LOGS( screen , row) ( ml_get_num_of_logged_lines( &(screen)->logs) + row)
 
@@ -110,7 +113,7 @@ receive_scrolled_out_line(
 	}
 	else
 	{
-		line->num_of_filled_chars = ml_get_num_of_filled_chars_except_spaces( line) ;
+		line->num_of_filled_chars = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 	}
 	
 	ml_log_add( &screen->logs , line) ;
@@ -119,6 +122,11 @@ receive_scrolled_out_line(
 	{
 		screen->backscroll_rows ++ ;
 	}
+
+	if( screen->search)
+	{
+		screen->search->row -- ;
+	}
 }
 
 static void
@@ -231,7 +239,7 @@ reverse_or_restore_color(
 		}
 		else
 		{
-			if( ( end_char_index = ml_get_num_of_filled_chars_except_spaces( line)) > 0)
+			if( ( end_char_index = ml_line_get_num_of_filled_chars_except_spaces( line)) > 0)
 			{
 				end_char_index -- ;
 			}
@@ -252,7 +260,7 @@ reverse_or_restore_color(
 			}
 		}
 
-		size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 		beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 
 		if( row > beg_row)
@@ -333,7 +341,7 @@ reverse_or_restore_color(
 				continue ;
 			}
 
-			size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+			size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 			beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 			
 			for( char_index = beg_regarding_rtl ;
@@ -349,7 +357,7 @@ reverse_or_restore_color(
 			return  1 ;
 		}
 
-		size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 		beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 
 		if( end_char_index < 0)
@@ -417,7 +425,7 @@ check_or_copy_region(
 		}
 		else
 		{
-			if( ( end_char_index = ml_get_num_of_filled_chars_except_spaces( line)) > 0)
+			if( ( end_char_index = ml_line_get_num_of_filled_chars_except_spaces( line)) > 0)
 			{
 				end_char_index -- ;
 			}
@@ -438,7 +446,7 @@ check_or_copy_region(
 			}
 		}
 
-		size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 		beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 
 		if( row > beg_row)
@@ -482,7 +490,7 @@ check_or_copy_region(
 					K_MAX(end_char_index,beg_regarding_rtl) ;
 			if( chars)
 			{
-				ml_line_copy_str( line , chars , end_char_index , count) ;
+				ml_line_copy_logical_str( line , chars , end_char_index , count) ;
 			}
 		}
 		else
@@ -491,7 +499,7 @@ check_or_copy_region(
 					K_MAX(beg_char_index,beg_regarding_rtl) ;
 			if( chars)
 			{
-				ml_line_copy_str( line , chars , beg_char_index , count) ;
+				ml_line_copy_logical_str( line , chars , beg_char_index , count) ;
 			}
 		}
 	}
@@ -502,7 +510,8 @@ check_or_copy_region(
 			count = (K_MIN(beg_char_index + 1,size_except_spaces) - beg_regarding_rtl) ;
 			if( chars)
 			{
-				ml_line_copy_str( line , chars , beg_regarding_rtl , count) ;
+				ml_line_copy_logical_str( line , chars ,
+					beg_regarding_rtl , count) ;
 			}
 		}
 		else
@@ -510,7 +519,7 @@ check_or_copy_region(
 			count = (size_except_spaces - K_MAX(beg_char_index,beg_regarding_rtl)) ;
 			if( chars)
 			{
-				ml_line_copy_str( line , chars ,
+				ml_line_copy_logical_str( line , chars ,
 					K_MAX(beg_char_index,beg_regarding_rtl) , count) ;
 			}
 		}
@@ -528,12 +537,13 @@ check_or_copy_region(
 		{
 			line = ml_screen_get_line( screen , row) ;
 
-			size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+			size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 			beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 
 			if( chars)
 			{
-				ml_line_copy_str( line , &chars[count] , beg_regarding_rtl ,
+				ml_line_copy_logical_str( line , &chars[count] ,
+					beg_regarding_rtl ,
 					size_except_spaces - beg_regarding_rtl) ;
 			}
 			count += (size_except_spaces - beg_regarding_rtl) ;
@@ -550,7 +560,7 @@ check_or_copy_region(
 
 		line = ml_screen_get_line( screen , row) ;
 
-		size_except_spaces = ml_get_num_of_filled_chars_except_spaces( line) ;
+		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
 		beg_regarding_rtl = ml_line_beg_char_index_regarding_rtl( line) ;
 
 		if( end_char_index < 0)
@@ -566,9 +576,10 @@ check_or_copy_region(
 		{
 			if( chars)
 			{
-				ml_line_copy_str( line , &chars[count] ,
+				ml_line_copy_logical_str( line , &chars[count] ,
 					K_MAX(end_char_index,beg_regarding_rtl) ,
-					size_except_spaces - K_MAX(end_char_index,beg_regarding_rtl)) ;
+					size_except_spaces -
+						K_MAX(end_char_index,beg_regarding_rtl)) ;
 			}
 			count += (size_except_spaces - K_MAX(end_char_index,beg_regarding_rtl)) ;
 		}
@@ -576,8 +587,10 @@ check_or_copy_region(
 		{
 			if( chars)
 			{
-				ml_line_copy_str( line , &chars[count] , beg_regarding_rtl ,
-					K_MIN(end_char_index + 1,size_except_spaces) - beg_regarding_rtl) ;
+				ml_line_copy_logical_str( line , &chars[count] ,
+					beg_regarding_rtl ,
+					K_MIN(end_char_index + 1,size_except_spaces) -
+						beg_regarding_rtl) ;
 			}
 			count += K_MIN(end_char_index + 1,size_except_spaces) ;
 		}
@@ -693,6 +706,8 @@ ml_screen_new(
 		goto  error3 ;
 	}
 
+	screen->search = NULL ;
+
 	screen->backscroll_rows = 0 ;
 	screen->backscroll_mode = bs_mode ;
 	screen->is_backscrolling = 0 ;
@@ -732,6 +747,8 @@ ml_screen_delete(
 
 	ml_log_final( &screen->logs) ;
 
+	ml_screen_search_final( screen) ;
+
 	free( screen) ;
 
 	return  1 ;
@@ -909,7 +926,7 @@ ml_screen_get_line(
 	int  row
 	)
 {
-	if( row < 0 - (int)ml_get_num_of_logged_lines( &screen->logs))
+	if( row < -(int)ml_get_num_of_logged_lines( &screen->logs))
 	{
 	#ifdef  __DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " row %d is over the beg of screen.\n" , row) ;
@@ -1152,7 +1169,7 @@ ml_screen_backscroll_to(
 	)
 {
 	ml_line_t *  line ;
-	int  count ;
+	u_int  count ;
 	
 	if( ! screen->is_backscrolling)
 	{
@@ -1253,7 +1270,7 @@ ml_screen_backscroll_downward(
 	)
 {
 	ml_line_t *  line ;
-	int  count ;
+	u_int  count ;
 	
 	if( ! screen->is_backscrolling)
 	{
@@ -1704,8 +1721,8 @@ ml_screen_insert_new_lines(
 	u_int  size
 	)
 {
-	int  count ;
-		
+	u_int  count ;
+
 	for( count = 0 ; count < size ; count ++)
 	{
 		ml_edit_insert_new_line( screen->edit) ;
@@ -1756,7 +1773,7 @@ ml_screen_delete_lines(
 	u_int  size
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -1855,7 +1872,7 @@ ml_screen_go_forward(
 	u_int  size
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -1878,7 +1895,7 @@ ml_screen_go_back(
 	u_int  size
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -1901,7 +1918,7 @@ ml_screen_go_upward(
 	u_int  size
 	)
 {
-	int  count ;
+	u_int  count ;
 	
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -1924,7 +1941,7 @@ ml_screen_go_downward(
 	u_int  size
 	)
 {
-	int  count ;
+	u_int  count ;
 	
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -2159,3 +2176,304 @@ ml_screen_is_backscrolling(
 		return  screen->is_backscrolling ;
 	}
 }
+
+int
+ml_screen_search_init(
+	ml_screen_t *  screen ,
+	int (*match)( size_t * , size_t * , void * , u_char * , int)
+	)
+{
+	if( screen->search)
+	{
+		return  0 ;
+	}
+
+	if( ! ( screen->search = malloc( sizeof( *screen->search))))
+	{
+		return  0 ;
+	}
+
+	screen->search->match = match ;
+
+	ml_screen_search_reset_position( screen) ;
+
+	return  1 ;
+}
+
+int
+ml_screen_search_final(
+	ml_screen_t *  screen
+	)
+{
+	free( screen->search) ;
+	screen->search = NULL ;
+
+	return  1 ;
+}
+
+int
+ml_screen_search_reset_position(
+	ml_screen_t *  screen
+	)
+{
+	if( ! screen->search)
+	{
+		return  0 ;
+	}
+
+	/* char_index == -1 has special meaning. */
+	screen->search->char_index = -1 ;
+	screen->search->row = -1 ;
+
+	return  1 ;
+}
+
+/*
+ * It is assumed that this function is called in *visual* context.
+ *
+ * XXX
+ * It is not supported to match text in multiple lines.
+ */
+int
+ml_screen_search_find(
+	ml_screen_t *  screen ,
+	int *  beg_char_index ,		/* visual position is returned */
+	int *  beg_row ,		/* visual position is returned */
+	int *  end_char_index ,		/* visual position is returned */
+	int *  end_row ,		/* visual position is returned */
+	void *  regex ,
+	int  backward
+	)
+{
+	ml_char_t *  line_str ;
+	mkf_parser_t *  parser ;
+	mkf_conv_t *  conv ;
+	u_char *  buf ;
+	size_t  buf_len ;
+	ml_line_t *  line ;
+	int  step ;
+	int  res ;
+
+	if( ! screen->search)
+	{
+		return  0 ;
+	}
+
+	if( ! ( line_str = ml_str_alloca( ml_screen_get_logical_cols( screen))))
+	{
+		return  0 ;
+	}
+
+	buf_len = ml_screen_get_logical_cols( screen) * MLCHAR_UTF_MAX_SIZE + 1 ;
+	if( ! ( buf = alloca( buf_len)))
+	{
+		return  0 ;
+	}
+
+	if( ! (parser = ml_str_parser_new()))
+	{
+		return  0 ;
+	}
+	
+	if( ! (conv = ml_conv_new( ML_UTF8)))
+	{
+		(*parser->delete)( parser) ;
+
+		return  0 ;
+	}
+
+	/* char_index == -1 has special meaning. */
+	if( screen->search->char_index == -1)
+	{
+		screen->search->char_index = ml_screen_cursor_char_index( screen) ;
+		screen->search->row = ml_screen_cursor_row( screen) ;
+	}
+
+	*beg_char_index = screen->search->char_index ;
+	*beg_row = screen->search->row ;
+	step = (backward ? -1 : 1) ;
+
+#ifdef  DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " Search start from %d %d\n" ,
+			*beg_char_index , *beg_row) ;
+#endif
+
+	res = 0 ;
+
+	for( ; (line = ml_screen_get_line( screen , *beg_row)) ; (*beg_row) += step)
+	{
+		size_t  line_len ;
+		size_t  match_beg ;
+		size_t  match_len ;
+
+		if( ( line_len = ml_line_get_num_of_filled_chars_except_spaces( line)) == 0)
+		{
+			continue ;
+		}
+
+		/*
+		 * Visual => Logical
+		 */
+		ml_line_copy_logical_str( line , line_str , 0 , line_len) ;
+
+		(*parser->init)( parser) ;
+		if( backward)
+		{
+			ml_str_parser_set_str( parser , line_str ,
+				(*beg_row != screen->search->row) ?
+					line_len : K_MIN(*beg_char_index + 1,line_len)) ;
+			*beg_char_index = 0 ;
+		}
+		else
+		{
+			if( *beg_row != screen->search->row)
+			{
+				*beg_char_index = 0 ;
+			}
+			else if( line_len <= *beg_char_index)
+			{
+				continue ;
+			}
+
+			ml_str_parser_set_str( parser , line_str + (*beg_char_index) ,
+					line_len - *beg_char_index) ;
+		}
+
+		(*conv->init)( conv) ;
+		*(buf + (*conv->convert)( conv , buf , buf_len - 1 , parser)) = '\0' ;
+
+		if( (*screen->search->match)( &match_beg , &match_len ,
+					regex , buf , backward))
+		{
+			size_t  count ;
+			size_t  comb_size ;
+			int  beg ;
+			int  end ;
+			int  meet_pos ;
+
+			ml_get_combining_chars( line_str + (*beg_char_index) , &comb_size) ;
+
+			for( count = 0 ; count < match_beg ; count++)
+			{
+				/* Ignore 2nd and following bytes. */
+				if( (buf[count] & 0xc0) != 0x80)
+				{
+					if( comb_size > 0)
+					{
+						comb_size -- ;
+					}
+					else if( (++ (*beg_char_index)) >= line_len - 1)
+					{
+						break ;
+					}
+					else
+					{
+						ml_get_combining_chars(
+							line_str + (*beg_char_index) ,
+							&comb_size) ;
+					}
+				}
+			}
+
+			*end_char_index = (*beg_char_index) - 1 ;
+			for( ; count < match_beg + match_len ; count++)
+			{
+				/* Ignore 2nd and following bytes. */
+				if( (buf[count] & 0xc0) != 0x80)
+				{
+					if( comb_size > 0)
+					{
+						comb_size -- ;
+					}
+					else if( (++ (*end_char_index)) >= line_len - 1)
+					{
+						break ;
+					}
+					else
+					{
+						ml_get_combining_chars(
+							line_str + (*end_char_index) ,
+							&comb_size) ;
+					}
+				}
+			}
+
+			if( *end_char_index < *beg_char_index)
+			{
+				continue ;
+			}
+
+			*end_row = *beg_row ;
+
+			if( backward)
+			{
+				if( *beg_char_index > 0)
+				{
+					screen->search->char_index = *beg_char_index - 1 ;
+					screen->search->row = *beg_row ;
+				}
+				else
+				{
+					screen->search->char_index =
+						ml_screen_get_logical_cols( screen) - 1 ;
+					screen->search->row = *beg_row - 1 ;
+				}
+			}
+			else
+			{
+				screen->search->char_index = *beg_char_index + 1 ;
+				screen->search->row = *beg_row ;
+			}
+
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " Search position x %d y %d\n" ,
+					screen->search->char_index , screen->search->row) ;
+		#endif
+
+			/*
+			 * Logical => Visual
+			 *
+			 * XXX Incomplete
+			 * Current implementation have problems like this.
+			 *  (logical)RRRLLLNNN => (visual)NNNLLLRRR
+			 *  Searching LLLNNN =>           ^^^^^^ hits but only NNNL is reversed.
+			 */
+			meet_pos = 0 ;
+			beg = ml_line_convert_logical_char_index_to_visual(
+					line , *beg_char_index , &meet_pos) ;
+			end = ml_line_convert_logical_char_index_to_visual(
+					line , *end_char_index , &meet_pos) ;
+			
+			if( beg > end)
+			{
+				*beg_char_index = end ;
+				*end_char_index = beg ;
+			}
+			else
+			{
+				*beg_char_index = beg ;
+				*end_char_index = end ;
+			}
+
+			if( ml_line_is_rtl( line))
+			{
+				int  char_index ;
+
+				/* XXX for x_selection */
+				char_index = -(*beg_char_index) ;
+				*beg_char_index = -(*end_char_index) ;
+				*end_char_index = char_index ;
+			}
+
+			res = 1 ;
+
+			break ;
+		}
+	}
+
+	(*parser->delete)( parser) ;
+	(*conv->delete)( conv) ;
+	ml_str_final( line_str , ml_screen_get_logical_cols( screen)) ;
+
+	return  res ;
+}
