@@ -24,7 +24,7 @@ ml_shape_t *  ml_arabic_shape_new(void) ;
 
 u_int16_t  ml_is_arabic_combining( ml_char_t *  prev2 , ml_char_t *  prev , ml_char_t *  ch) ;
 
-ml_shape_t *  ml_iscii_shape_new( ml_iscii_lang_t  iscii_lang) ;
+ml_shape_t *  ml_iscii_shape_new( void) ;
 
 
 #endif
