@@ -29,11 +29,11 @@ ml_term_new(
 	int  use_multi_col_char ,
 	int  use_bidi ,
 	ml_bidi_mode_t  bidi_mode ,
+	int  use_ind ,
 	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
-	ml_vertical_mode_t  vertical_mode ,
-	mkf_iscii_lang_t  iscii_lang_type
+	ml_vertical_mode_t  vertical_mode
 	)
 {
 	ml_term_t *  term ;
@@ -80,11 +80,10 @@ ml_term_new(
 	}
 
 	term->shape = NULL ;
-	term->iscii_lang = NULL ;
-	term->iscii_lang_type = iscii_lang_type ;
 	term->vertical_mode = vertical_mode ;
 	term->bidi_mode = bidi_mode ;
 	term->use_bidi = use_bidi ;
+	term->use_ind = use_ind ;
 	term->use_dynamic_comb = use_dynamic_comb ;
 
 	term->is_auto_encoding = is_auto_encoding ;
@@ -92,12 +91,13 @@ ml_term_new(
 	term->win_name = NULL ;
 	term->icon_name = NULL ;
 	term->icon_path = NULL ;
+
 	term->is_mouse_pos_sending = 0 ;
 	term->is_app_keypad = 0 ;
 	term->is_app_cursor_keys = 0 ;
 
 	term->is_attached = 0 ;
-	
+
 	return  term ;
 
 error:
@@ -131,11 +131,6 @@ ml_term_delete(
 		(*term->shape->delete)( term->shape) ;
 	}
 
-	if( term->iscii_lang)
-	{
-		ml_iscii_lang_delete( term->iscii_lang) ;
-	}
-
 	free( term->win_name) ;
 	free( term->icon_name) ;
 	free( term->icon_path) ;
@@ -144,7 +139,7 @@ ml_term_delete(
 	ml_vt100_parser_delete( term->parser) ;
 
 	ml_config_menu_final( &term->config_menu) ;
-	
+
 	free( term) ;
 
 	return  1 ;
@@ -170,13 +165,13 @@ ml_term_zombie(
 int
 ml_term_open_pty(
 	ml_term_t *  term ,
-	char *  cmd_path ,
+	const char *  cmd_path ,
 	char **  argv ,
 	char **  env ,
-	char *  host ,
-	char *  pass ,
-	char *  pubkey ,
-	char *  privkey
+	const char *  host ,
+	const char *  pass ,
+	const char *  pubkey ,
+	const char *  privkey
 	)
 {
 	if( term->pty)
@@ -281,31 +276,6 @@ ml_term_is_attached(
 	return  term->is_attached ;
 }
 
-int
-ml_term_parse_vt100_sequence(
-	ml_term_t *  term
-	)
-{
-	return  ml_parse_vt100_sequence( term->parser) ;
-}
-
-int
-ml_term_change_encoding(
-	ml_term_t *  term ,
-	ml_char_encoding_t  encoding
-	)
-{
-	return  ml_vt100_parser_change_encoding( term->parser , encoding) ;
-}
-
-ml_char_encoding_t
-ml_term_get_encoding(
-	ml_term_t *  term
-	)
-{
-	return  ml_vt100_parser_get_encoding( term->parser) ;
-}
-
 int
 ml_term_set_auto_encoding(
 	ml_term_t *  term ,
@@ -325,42 +295,6 @@ ml_term_is_auto_encoding(
 	return  term->is_auto_encoding ;
 }
 
-int
-ml_term_set_unicode_policy(
-	ml_term_t *  term ,
-	ml_unicode_policy_t  policy
-	)
-{
-	return  ml_vt100_parser_set_unicode_policy( term->parser , policy) ;
-}
-
-size_t
-ml_term_convert_to(
-	ml_term_t *  term ,
-	u_char *  dst ,
-	size_t  len ,
-	mkf_parser_t *  parser
-	)
-{
-	return  ml_vt100_parser_convert_to( term->parser , dst , len , parser) ;
-}
-
-int
-ml_term_init_encoding_parser(
-	ml_term_t *  term
-	)
-{
-	return  ml_init_encoding_parser( term->parser) ;
-}
-
-int
-ml_term_init_encoding_conv(
-	ml_term_t *  term
-	)
-{
-	return  ml_init_encoding_conv( term->parser) ;
-}
-
 int
 ml_term_set_logging_vt_seq(
 	ml_term_t *  term ,
@@ -467,38 +401,6 @@ ml_term_resize(
 	return  1 ;
 }
 
-int
-ml_term_cursor_col(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_cursor_col( term->screen) ;
-}
-
-int
-ml_term_cursor_char_index(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_cursor_char_index( term->screen) ;
-}
-
-int
-ml_term_cursor_row(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_cursor_row( term->screen) ;
-}
-
-int
-ml_term_cursor_row_in_screen(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_cursor_row_in_screen( term->screen) ;
-}
-
 int
 ml_term_unhighlight_cursor(
 	ml_term_t *  term ,
@@ -531,106 +433,6 @@ ml_term_unhighlight_cursor(
 	return  ret ;
 }
 
-u_int
-ml_term_get_cols(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_cols( term->screen) ;
-}
-
-u_int
-ml_term_get_rows(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_rows( term->screen) ;
-}
-
-u_int
-ml_term_get_logical_cols(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_logical_cols( term->screen) ;
-}
-
-u_int
-ml_term_get_logical_rows(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_logical_rows( term->screen) ;
-}
-
-u_int
-ml_term_get_log_size(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_log_size( term->screen) ;
-}
-
-int
-ml_term_change_log_size(
-	ml_term_t *  term ,
-	u_int  log_size
-	)
-{
-	return  ml_screen_change_log_size( term->screen , log_size) ;
-}
-
-u_int
-ml_term_get_num_of_logged_lines(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_num_of_logged_lines( term->screen) ;
-}
-
-int
-ml_term_convert_scr_row_to_abs(
-	ml_term_t *  term ,
-	int  row
-	)
-{
-	return  ml_screen_convert_scr_row_to_abs( term->screen , row) ;
-}
-
-ml_line_t *
-ml_term_get_line(
-	ml_term_t *  term ,
-	int  row
-	)
-{
-	return  ml_screen_get_line( term->screen , row) ;
-}
-
-ml_line_t *
-ml_term_get_line_in_screen(
-	ml_term_t *  term ,
-	int  row
-	)
-{
-	return  ml_screen_get_line_in_screen( term->screen , row) ;
-}
-
-ml_line_t *
-ml_term_get_cursor_line(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_cursor_line( term->screen) ;
-}
-
-int
-ml_term_is_cursor_visible(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_is_cursor_visible( term->screen) ;
-}
-
 /*
  * Not implemented yet.
  */
@@ -691,7 +493,10 @@ ml_term_set_modified_lines(
 	int  row ;
 	ml_line_t *  line ;
 
-	ml_screen_logical( term->screen) ;
+	if( term->vertical_mode)
+	{
+		ml_screen_logical( term->screen) ;
+	}
 
 	for( row = beg ; row <= end ; row ++)
 	{
@@ -701,8 +506,11 @@ ml_term_set_modified_lines(
 		}
 	}
 
-	/* ml_screen_render( term->screen) ; */
-	ml_screen_visual( term->screen) ;
+	if( term->vertical_mode)
+	{
+		/* ml_screen_render( term->screen) ; */
+		ml_screen_visual( term->screen) ;
+	}
 
 	return  1 ;
 }
@@ -717,7 +525,10 @@ ml_term_set_modified_lines_in_screen(
 	int  row ;
 	ml_line_t *  line ;
 
-	ml_screen_logical( term->screen) ;
+	if( term->vertical_mode)
+	{
+		ml_screen_logical( term->screen) ;
+	}
 
 	for( row = beg ; row <= end ; row ++)
 	{
@@ -727,8 +538,11 @@ ml_term_set_modified_lines_in_screen(
 		}
 	}
 
-	/* ml_screen_render( term->screen) ; */
-	ml_screen_visual( term->screen) ;
+	if( term->vertical_mode)
+	{
+		/* ml_screen_render( term->screen) ; */
+		ml_screen_visual( term->screen) ;
+	}
 
 	return  1 ;
 }
@@ -738,10 +552,18 @@ ml_term_set_modified_all_lines_in_screen(
 	ml_term_t *  term
 	)
 {
-	ml_screen_logical( term->screen) ;
+	if( term->vertical_mode)
+	{
+		ml_screen_logical( term->screen) ;
+	}
+
 	ml_screen_set_modified_all( term->screen) ;
-	/* ml_screen_render( term->screen) ; */
-	ml_screen_visual( term->screen) ;
+
+	if( term->vertical_mode)
+	{
+		/* ml_screen_render( term->screen) ; */
+		ml_screen_visual( term->screen) ;
+	}
 
 	return  1 ;
 }
@@ -754,7 +576,10 @@ ml_term_updated_all(
 	int  row ;
 	ml_line_t *  line ;
 
-	ml_screen_logical( term->screen) ;
+	if( term->vertical_mode)
+	{
+		ml_screen_logical( term->screen) ;
+	}
 
 	for( row = 0 ; row < ml_edit_get_rows( term->screen->edit) ; row ++)
 	{
@@ -764,8 +589,11 @@ ml_term_updated_all(
 		}
 	}
 
-	/* ml_screen_render( term->screen) ; */
-	ml_screen_visual( term->screen) ;
+	if( term->vertical_mode)
+	{
+		/* ml_screen_render( term->screen) ; */
+		ml_screen_visual( term->screen) ;
+	}
 
 	return  1 ;
 }
@@ -790,30 +618,18 @@ ml_term_update_special_visual(
 		term->shape = NULL ;
 	}
 
-	if( term->iscii_lang)
-	{
-		ml_iscii_lang_delete( term->iscii_lang) ;
-		term->iscii_lang = NULL ;
-	}
-
 	term->screen->use_dynamic_comb = 0 ;
 	had_logvis = ml_screen_delete_logical_visual( term->screen) ;
 
-	if( ml_term_get_encoding( term) == ML_ISCII
-	#ifdef  USE_IND
-	    /* XXX */
-	    || (ml_term_get_encoding( term) == ML_UTF8 && ! term->use_bidi &&
-	        (term->parser->unicode_policy & NOT_USE_UNICODE_FONT))
-	#endif
-	    )
+	if( IS_ISCII_ENCODING( ml_term_get_encoding( term))
+	    || (ml_term_get_encoding( term) == ML_UTF8 && ! term->use_bidi && term->use_ind) )
 	{
 		/*
 		 * It is impossible to process ISCII with other special visuals.
 		 */
 
-		if( ( term->iscii_lang = ml_iscii_lang_new( term->iscii_lang_type)) &&
-		    ( term->shape = ml_iscii_shape_new( term->iscii_lang)) &&
-		    ( logvis = ml_logvis_iscii_new( term->iscii_lang)))
+		if( ( term->shape = ml_iscii_shape_new()) &&
+		    ( logvis = ml_logvis_iscii_new()))
 		{
 			if( ml_screen_add_logical_visual( term->screen , logvis))
 			{
@@ -833,7 +649,7 @@ ml_term_update_special_visual(
 		else
 		{
 			kik_warn_printf( KIK_DEBUG_TAG
-				" ml_iscii_xxx_new()/ml_logvis_iscii_new() failed.\n") ;
+				" ml_iscii_shape_new()/ml_logvis_iscii_new() failed.\n") ;
 		}
 	#endif
 	}
@@ -932,12 +748,6 @@ ml_term_update_special_visual(
 			term->shape = NULL ;
 		}
 
-		if( term->iscii_lang)
-		{
-			ml_iscii_lang_delete( term->iscii_lang) ;
-			term->iscii_lang = NULL ;
-		}
-
 		term->screen->use_dynamic_comb = 0 ;
 		ml_screen_delete_logical_visual( term->screen) ;
 	}
@@ -955,23 +765,6 @@ ml_term_update_special_visual(
 	}
 }
 
-ml_bs_mode_t
-ml_term_is_backscrolling(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_is_backscrolling( term->screen) ;
-}
-
-int
-ml_term_set_backscroll_mode(
-	ml_term_t *  term ,
-	ml_bs_mode_t  mode
-	)
-{
-	return  ml_set_backscroll_mode( term->screen , mode) ;
-}
-
 int
 ml_term_enter_backscroll_mode(
 	ml_term_t *  term
@@ -988,140 +781,6 @@ ml_term_enter_backscroll_mode(
 	return  ml_enter_backscroll_mode( term->screen) ;
 }
 
-int
-ml_term_exit_backscroll_mode(
-	ml_term_t *  term
-	)
-{
-	return  ml_exit_backscroll_mode( term->screen) ;
-}
-
-int
-ml_term_backscroll_to(
-	ml_term_t *  term ,
-	int  row
-	)
-{
-	return  ml_screen_backscroll_to( term->screen , row) ;
-}
-
-int
-ml_term_backscroll_upward(
-	ml_term_t *  term ,
-	u_int  size
-	)
-{
-	return  ml_screen_backscroll_upward( term->screen , size) ;
-}
-
-int
-ml_term_backscroll_downward(
-	ml_term_t *  term ,
-	u_int  size
-	)
-{
-	return  ml_screen_backscroll_downward( term->screen , size) ;
-}
-
-u_int
-ml_term_get_tab_size(
-	ml_term_t *  term
-	)
-{
-	return  ml_screen_get_tab_size( term->screen) ;
-}
-
-int
-ml_term_set_tab_size(
-	ml_term_t *  term ,
-	u_int  tab_size
-	)
-{
-	return  ml_screen_set_tab_size( term->screen , tab_size) ;
-}
-
-int
-ml_term_reverse_color(
-	ml_term_t *  term ,
-	int  beg_char_index ,
-	int  beg_row ,
-	int  end_char_index ,
-	int  end_row
-	)
-{
-	return  ml_screen_reverse_color( term->screen , beg_char_index , beg_row ,
-			end_char_index , end_row) ;
-}
-
-int
-ml_term_restore_color(
-	ml_term_t *  term ,
-	int  beg_char_index ,
-	int  beg_row ,
-	int  end_char_index ,
-	int  end_row
-	)
-{
-	return  ml_screen_restore_color( term->screen , beg_char_index , beg_row ,
-			end_char_index , end_row) ;
-}
-
-u_int
-ml_term_copy_region(
-	ml_term_t *  term ,
-	ml_char_t *  chars ,
-	u_int  num_of_chars ,
-	int  beg_char_index ,
-	int  beg_row ,
-	int  end_char_index ,
-	int  end_row
-	)
-{
-	return  ml_screen_copy_region( term->screen , chars , num_of_chars ,
-			beg_char_index , beg_row , end_char_index , end_row) ;
-}
-
-u_int
-ml_term_get_region_size(
-	ml_term_t *  term ,
-	int  beg_char_index ,
-	int  beg_row ,
-	int  end_char_index ,
-	int  end_row
-	)
-{
-	return  ml_screen_get_region_size( term->screen , beg_char_index , beg_row ,
-			end_char_index , end_row) ;
-}
-
-int
-ml_term_get_line_region(
-	ml_term_t *  term ,
-	int *  beg_row ,
-	int *  end_char_index ,
-	int *  end_row ,
-	int  base_row
-	)
-{
-	return  ml_screen_get_line_region( term->screen , beg_row , end_char_index ,
-			end_row , base_row) ;
-}
-
-int
-ml_term_get_word_region(
-	ml_term_t *  term ,
-	int *  beg_char_index ,
-	int *  beg_row ,
-	int *  end_char_index ,
-	int *  end_row ,
-	int  base_char_index ,
-	int  base_row
-	)
-{
-	return  ml_screen_get_word_region( term->screen , beg_char_index , beg_row ,
-			end_char_index , end_row , base_char_index , base_row) ;
-}
-
 int
 ml_term_set_char_combining_flag(
 	ml_term_t *  term ,
@@ -1160,23 +819,6 @@ ml_term_is_using_multi_col_char(
 	return  term->parser->use_multi_col_char ;
 }
 
-int
-ml_term_set_col_size_of_width_a(
-	ml_term_t *  term ,
-	u_int  col_size_a
-	)
-{
-	return  ml_vt100_parser_set_col_size_of_width_a( term->parser , col_size_a) ;
-}
-
-u_int
-ml_term_get_col_size_of_width_a(
-	ml_term_t *  term
-	)
-{
-	return  ml_vt100_parser_get_col_size_of_width_a( term->parser) ;
-}
-
 int
 ml_term_set_mouse_report(
 	ml_term_t *  term ,
