@@ -509,7 +509,7 @@ save_cursor(
 {
 	ml_vt100_storable_states_t *  dest ;
 
-	dest = (ml_screen_is_alternative_edit(  vt100_parser->screen) ) ?
+	dest = (ml_screen_is_alternative_edit( vt100_parser->screen)) ?
 		&(vt100_parser->saved_alternate)
 		: &(vt100_parser->saved_normal) ;
 	dest->is_saved = 1 ;
@@ -676,6 +676,35 @@ set_icon_name(
 	}
 }
 
+static void
+switch_im_mode(
+	ml_vt100_parser_t *  vt100_parser
+	)
+{
+	if( HAS_XTERM_LISTENER(vt100_parser,switch_im_mode))
+	{
+		(*vt100_parser->xterm_listener->switch_im_mode)(
+					vt100_parser->xterm_listener->self) ;
+	}
+}
+
+static int
+im_is_active(
+	ml_vt100_parser_t *  vt100_parser
+	)
+{
+	if( HAS_XTERM_LISTENER(vt100_parser,im_is_active))
+	{
+		return  (*vt100_parser->xterm_listener->im_is_active)(
+					vt100_parser->xterm_listener->self) ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
+
 /*
  * This function will destroy the content of pt.
  */
@@ -713,12 +742,9 @@ config_protocol_set(
 					vt100_parser->config_listener->self , dev , key , val) ;
 			}
 
-			/* XXX */
-			if( vt100_parser->config_listener == NULL)
+			if( ! vt100_parser->config_listener)
 			{
-				/*
-				 * pty changed.
-				 */
+				/* pty changed. */
 				break ;
 			}
 		}
@@ -808,8 +834,15 @@ config_protocol_get(
 		char *  dev ;
 		char *  key ;
 		int  ret ;
-		
+
+		/*
+		 * It is assumed that screen is not redrawn not in
+		 * vt100_parser->config_listener->get, so vt100_parser->config_listener->get
+		 * is not held between stop_vt100_cmd and start_vt100_cmd.
+		 */
+	#if  0
 		stop_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		ret = ml_parse_proto( &dev , &key , NULL , &pt , to_menu == 0) ;
 		if( ret == -1)
@@ -830,13 +863,15 @@ config_protocol_get(
 		}
 		else /* if( ret < 0) */
 		{
-			char  msg[] = "error" ;
+			char *  msg = "error" ;
 			
 			(*vt100_parser->config_listener->get)(
 				vt100_parser->config_listener->self , NULL , msg , to_menu) ;
 		}
 
+	#if  0
 		start_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		return  1 ;
 	}
@@ -862,15 +897,25 @@ config_protocol_set_font(
 		char *  key ;
 		char *  val ;
 
+		/*
+		 * Screen is redrawn not in vt100_parser->config_listener->set_font
+		 * but in stop_vt100_cmd, so it is not necessary to hold
+		 * vt100_parser->config_listener->set_font between stop_vt100_cmd and
+		 * start_vt100_cmd.
+		 */
+	#if  0
 		stop_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		if( ml_parse_proto2( &file , &key , &val , pt , 0) && key && val)
 		{
 			(*vt100_parser->config_listener->set_font)(
-				vt100_parser->config_listener->self , file , key , val, save) ;
+				vt100_parser->config_listener->self , file , key , val , save) ;
 		}
 
+	#if  0
 		start_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		return  1 ;
 	}
@@ -897,7 +942,9 @@ config_protocol_get_font(
 		char *  cs ;
 		int  ret ;
 
+	#if  0
 		stop_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		ret = ml_parse_proto2( &file , &key , NULL , pt , to_menu == 0) ;
 		if( ret == -1)
@@ -929,7 +976,9 @@ config_protocol_get_font(
 				NULL , msg , msg , to_menu) ;
 		}
 
+	#if  0
 		start_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		return  1 ;
 	}
@@ -955,15 +1004,25 @@ config_protocol_set_color(
 		char *  key ;
 		char *  val ;
 
+		/*
+		 * Screen is redrawn not in vt100_parser->config_listener->set_color
+		 * but in stop_vt100_cmd, so it is not necessary to hold
+		 * vt100_parser->config_listener->set_font between stop_vt100_cmd and
+		 * start_vt100_cmd.
+		 */
+	#if  0
 		stop_vt100_cmd( vt100_parser , 0) ;
-
+	#endif
+	
 		if( ml_parse_proto2( &file , &key , &val , pt , 0) && key && val)
 		{
 			(*vt100_parser->config_listener->set_color)(
 				vt100_parser->config_listener->self , file , key , val, save) ;
 		}
 
+	#if  0
 		start_vt100_cmd( vt100_parser , 0) ;
+	#endif
 
 		return  1 ;
 	}
@@ -1158,6 +1217,11 @@ soft_reset(
 	change_char_attr( vt100_parser , 0) ;
 
 	ml_init_encoding_parser( vt100_parser) ;
+
+	( ml_screen_is_alternative_edit( vt100_parser->screen) ?
+		&vt100_parser->saved_alternate
+		: &vt100_parser->saved_normal)->is_saved = 0 ;
+	vt100_parser->im_is_active = 0 ;
 }
 
 
@@ -1927,6 +1991,35 @@ parse_vt100_escape_sequence(
 					soft_reset( vt100_parser) ;
 				}
 			}
+			else if( pre_ch == '<')
+			{
+				/* Teraterm compatible IME control sequence */
+
+				if( *str_p == 'r')
+				{
+					/* Restore IME state */
+					if( vt100_parser->im_is_active !=
+						im_is_active( vt100_parser))
+					{
+						switch_im_mode( vt100_parser) ;
+					}
+				}
+				else if( *str_p == 's')
+				{
+					/* Save IME state */
+
+					vt100_parser->im_is_active = im_is_active( vt100_parser) ;
+				}
+				else if( *str_p == 't')
+				{
+					/* ps[0] = 0 (Close), ps[0] = 1 (Open) */
+
+					if( ps[0] != im_is_active( vt100_parser))
+					{
+						switch_im_mode( vt100_parser) ;
+					}
+				}
+			}
 			/* Other pre_ch(0x20-0x2f or 0x3a-0x3f) */
 			else if( pre_ch)
 			{
@@ -3004,6 +3097,24 @@ parse_vt100_sequence(
 
 					ch = non_ucs ;
 				}
+			#ifdef  USE_IND
+				else
+				{
+					mkf_char_t  non_ucs ;
+
+					/* XXX */
+					if( mkf_map_ucs4_to_iscii( &non_ucs ,
+						mkf_bytes_to_int( ch.ch , ch.size)))
+					{
+						if( vt100_parser->unicode_policy &
+							USE_UNICODE_PROPERTY)
+						{
+							non_ucs.property = ch.property ;
+						}
+						ch = non_ucs ;
+					}
+				}
+			#endif
 			}
 			else if( ( /* ONLY_USE_UNICODE_FONT or USE_UNICODE_PROPERTY */
 				   vt100_parser->unicode_policy > NOT_USE_UNICODE_FONT &&
@@ -3356,7 +3467,7 @@ ml_parse_vt100_sequence(
 	while(  /* (PTY_RD_BUFFER_SIZE / 2) is baseless. */
 		vt100_parser->len >= (PTY_RD_BUFFER_SIZE / 2) &&
 		(++count) < 3 && receive_bytes( vt100_parser)) ;
-	
+
 	stop_vt100_cmd( vt100_parser , 1) ;
 
 	return  1 ;
