@@ -28,7 +28,6 @@
 #include  "mc_sb.h"
 #include  "mc_im.h"
 #include  "mc_check.h"
-#include  "mc_iscii_lang.h"
 #include  "mc_sb_view.h"
 #include  "mc_io.h"
 #include  "mc_pty.h"
@@ -57,6 +56,23 @@ end_application(
 	return  FALSE ;
 }
 
+
+static gint
+bidi_flag_checked(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	GtkWidget *  ind_flag ;
+
+	ind_flag = data ;
+
+	gtk_widget_set_sensitive( ind_flag , ! GTK_TOGGLE_BUTTON(widget)->active) ;
+
+	return  1 ;
+}
+
+
 /*
  *  ********  procedures when buttons are clicked  ********
  */
@@ -67,7 +83,6 @@ update(
 	)
 {
     mc_update_char_encoding() ;
-    mc_update_iscii_lang() ;
     mc_update_color(MC_COLOR_FG) ;
     mc_update_bgtype() ;
     mc_update_color(MC_COLOR_SBFG) ;
@@ -94,6 +109,7 @@ update(
     mc_update_flag_mode(MC_FLAG_RECVUCS);
     mc_update_flag_mode(MC_FLAG_MCOL);
     mc_update_flag_mode(MC_FLAG_BIDI);
+    mc_update_flag_mode(MC_FLAG_IND);
     mc_update_flag_mode(MC_FLAG_AWIDTH);
     mc_update_flag_mode(MC_FLAG_CLIPBOARD);
 
@@ -206,7 +222,7 @@ pty_button_clicked(
  */
 
 static void
-addbutton(char *label, gint (func)(GtkWidget *, gpointer), GtkWidget *box)
+addbutton(const char *label, gint (func)(GtkWidget *, gpointer), GtkWidget *box)
 {
 	GtkWidget *button;
 	button = gtk_button_new_with_label(label);
@@ -323,6 +339,7 @@ show(void)
 	GtkWidget *  frame ;
 	GtkWidget *  label ;
 	GtkWidget *  config_widget ;
+	GtkWidget *  bidi_flag ;
 	GtkWidget *  separator ;
 	
 	window = gtk_window_new( GTK_WINDOW_TOPLEVEL) ;
@@ -377,14 +394,6 @@ show(void)
 	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
 
 
-	if (!(config_widget = mc_iscii_lang_config_widget_new())) return 0;
-#ifndef USE_IND
-	gtk_widget_set_sensitive(config_widget, 0);
-#endif
-	gtk_widget_show(config_widget);
-	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
-
-
 	if (!(config_widget = mc_im_config_widget_new())) return 0;
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
@@ -394,14 +403,28 @@ show(void)
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-	if (!(config_widget = mc_flag_config_widget_new(MC_FLAG_BIDI)))
+	if (!(bidi_flag = mc_flag_config_widget_new(MC_FLAG_BIDI)))
 	    return 0;
 #ifndef USE_FRIBIDI
-	gtk_widget_set_sensitive(config_widget, 0);
+	gtk_widget_set_sensitive(bidi_flag, 0);
 #endif
+	gtk_widget_show(bidi_flag);
+	gtk_box_pack_start(GTK_BOX(hbox), bidi_flag, TRUE, TRUE, 0);
+
+	if (!(config_widget = mc_flag_config_widget_new(MC_FLAG_IND)))
+	    return 0;
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(hbox), config_widget, TRUE, TRUE, 0);
 
+#ifdef  USE_IND
+#ifdef  USE_FRIBIDI
+	gtk_signal_connect(GTK_OBJECT(bidi_flag), "toggled" ,
+		GTK_SIGNAL_FUNC(bidi_flag_checked), config_widget);
+	gtk_widget_set_sensitive(config_widget, ! GTK_TOGGLE_BUTTON(bidi_flag)->active);
+#endif
+#else
+	gtk_widget_set_sensitive(config_widget, 0);
+#endif
 
 	if(!(config_widget = mc_flag_config_widget_new(MC_FLAG_COMB)))
 	    return 0;
