@@ -1,5 +1,5 @@
 /*
- *	$Id: mc_io.h,v 1.5 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id$
  */
 
 #ifndef  __MC_IO_H__
@@ -10,7 +10,13 @@ typedef enum {
 	mc_io_set      = 5379,
 	mc_io_get      = 5381,
 	mc_io_save     = 5382,
-	mc_io_set_save = 5383
+	mc_io_set_save = 5383,
+	mc_io_set_font = 5384,
+	mc_io_get_font = 5386,
+	mc_io_set_save_font = 5387,
+	mc_io_set_color = 5388,
+	mc_io_get_color = 5390,
+	mc_io_set_save_color = 5391,
 } mc_io_t;
 
 
@@ -24,7 +30,12 @@ char *  mc_get_str_value( char *  key) ;
 
 int  mc_get_flag_value( char *  key) ;
 
-char *  mc_get_bgtype(void) ;
+int  mc_gui_is_win32(void) ;
+
+int  mc_set_font_name( mc_io_t  io , char *  file , char *  font_size , char *  cs ,
+	char *  font_name) ;
+
+char *  mc_get_font_name( char *  file , char *  font_size , char *  cs) ;
 
 
 #endif
