@@ -1,5 +1,5 @@
 /*
- *	$Id: x_font_config.h,v 1.4 2003/12/27 09:06:22 arakiken Exp $
+ *	$Id$
  */
 
 #ifndef  __X_FONT_CONFIG_H__
@@ -16,11 +16,14 @@ KIK_MAP_TYPEDEF( x_font_name , ml_font_t , char *) ;
 
 typedef struct  x_font_config
 {
-	/* public(readonly) */
+	/* Public(readonly) */
 	x_type_engine_t  type_engine ;
 	x_font_present_t  font_present ;
 
-	/* private */
+	/*
+	 * Private
+	 * font_configs whose difference is only FONT_AA share these members.
+	 */
 	KIK_MAP( x_font_name) *  font_name_table ;
 	KIK_MAP( x_font_name)  default_font_name_table ;
 
@@ -47,10 +50,14 @@ int  x_customize_font_name( x_font_config_t *  font_config , ml_font_t  font ,
 	char *  fontname , u_int  font_size) ;
 
 int  x_customize_default_font_name( x_font_config_t *  font_config , ml_font_t  font ,
-	char *  fontname , u_int  font_size) ;
+	char *  fontname) ;
+
+int  x_customize_font_file( const char *  file, char *  key, char *  value, int  save) ;
 
 char *  x_get_config_font_name( x_font_config_t *  font_config , u_int  font_size , ml_font_t  font) ;
 
+char *  x_get_config_font_name2( const char *  file , u_int  font_size , char *  font_cs) ;
+
 char *  x_get_all_config_font_names( x_font_config_t *  font_config , u_int  font_size) ;
 
 
