@@ -49,7 +49,17 @@ static encoding_to_cs_table_t  usascii_font_cs_table[] =
 	{ ML_ISO8859_16 , ISO8859_16_R } ,
 	{ ML_TCVN5712 , TCVN5712_3_1993 } ,
 	
-	{ ML_ISCII , ISO8859_1_R } ,
+	{ ML_ISCII_ASSAMESE , ISO8859_1_R } ,
+	{ ML_ISCII_BENGALI , ISO8859_1_R } ,
+	{ ML_ISCII_GUJARATI , ISO8859_1_R } ,
+	{ ML_ISCII_HINDI , ISO8859_1_R } ,
+	{ ML_ISCII_KANNADA , ISO8859_1_R } ,
+	{ ML_ISCII_MALAYALAM , ISO8859_1_R } ,
+	{ ML_ISCII_ORIYA , ISO8859_1_R } ,
+	{ ML_ISCII_PUNJABI , ISO8859_1_R } ,
+	{ ML_ISCII_ROMAN , ISO8859_1_R } ,
+	{ ML_ISCII_TAMIL , ISO8859_1_R } ,
+	{ ML_ISCII_TELUGU , ISO8859_1_R } ,
 	{ ML_VISCII , VISCII } ,
 	{ ML_KOI8_R , KOI8_R } ,
 	{ ML_KOI8_U , KOI8_U } ,
@@ -85,26 +95,6 @@ change_font_cache(
 	return  1 ;
 }
 
-static int
-release_font_config(
-	x_font_manager_t *  font_man
-	)
-{
-	if( font_man->is_local_font_config)
-	{
-		x_font_config_delete( font_man->font_config) ;
-		font_man->is_local_font_config = 0 ;
-	}
-	else
-	{
-		x_release_font_config( font_man->font_config) ;
-	}
-
-	font_man->font_config = NULL ;
-
-	return  1 ;
-}
-
 
 /* --- global functions --- */
 
@@ -153,8 +143,6 @@ x_font_manager_new(
 		return  NULL ;
 	}
 
-	font_man->is_local_font_config = 0 ;
-	
 	font_man->usascii_font_cs_changable = usascii_font_cs_changable ;
 	if(  x_get_max_font_size() - x_get_min_font_size() >= step_in_changing_font_size)
 	{
@@ -175,7 +163,8 @@ x_font_manager_delete(
 {
 	x_release_font_cache( font_man->font_cache) ;
 
-	release_font_config( font_man) ;
+	x_release_font_config( font_man->font_config) ;
+	font_man->font_config = NULL ;
 	
 	free( font_man) ;
 
@@ -262,9 +251,14 @@ x_change_font_present(
 	{
 		font_present &= ~FONT_AA ;
 	}
-	else if( ( font_present & FONT_AA) && ( font_man->font_config->type_engine == TYPE_XCORE))
+	else if( ( font_present & FONT_AA) &&
+		font_man->font_config->type_engine == TYPE_XCORE && type_engine == TYPE_XCORE)
 	{
+	#ifdef  USE_TYPE_XFT
 		type_engine = TYPE_XFT ;
+	#else
+		type_engine = TYPE_CAIRO ;
+	#endif
 	}
 #endif
 
@@ -294,7 +288,7 @@ x_change_font_present(
 
 	change_font_cache( font_man , font_cache) ;
 
-	release_font_config( font_man) ;
+	x_release_font_config( font_man->font_config) ;
 	font_man->font_config = font_config ;
 
 	return  1 ;
@@ -565,68 +559,3 @@ x_get_usascii_font_cs(
 		return  usascii_font_cs_table[encoding].cs ;
 	}
 }
-
-int
-x_activate_local_font_config(
-	x_font_manager_t *  font_man ,
-	x_font_config_t *  font_config		/* Should be generated by x_font_config_new() */
-	)
-{
-	x_font_cache_t *  font_cache ;
-
-	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
-				font_man->font_cache->font_size ,
-				font_man->font_cache->usascii_font_cs ,
-				font_config , font_man->font_cache->use_multi_col_char ,
-				font_man->font_cache->letter_space)) == NULL)
-	{
-		return  0 ;
-	}
-
-	change_font_cache( font_man , font_cache) ;
-	
-	release_font_config( font_man) ;
-	font_man->font_config = font_config ;
-	font_man->is_local_font_config = 1 ;
-
-	return  1 ;
-}
-
-int
-x_deactivate_local_font_config(
-	x_font_manager_t *  font_man
-	)
-{
-	x_font_config_t *  font_config ;
-	x_font_cache_t *  font_cache ;
-
-	if( ! font_man->is_local_font_config)
-	{
-		return  0 ;
-	}
-
-	if( ( font_config = x_acquire_font_config( font_man->font_config->type_engine ,
-				font_man->font_config->font_present)) == NULL)
-	{
-		return  0 ;
-	}
-	
-	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
-				font_man->font_cache->font_size ,
-				font_man->font_cache->usascii_font_cs , font_config ,
-				font_man->font_cache->use_multi_col_char ,
-				font_man->font_cache->letter_space)) == NULL)
-	{
-		x_release_font_config( font_config) ;
-		
-		return  0 ;
-	}
-
-	change_font_cache( font_man , font_cache) ;
-
-	x_font_config_delete( font_man->font_config) ;
-	font_man->font_config = font_config ;
-	font_man->is_local_font_config = 0 ;
-
-	return  1 ;
-}
