@@ -35,8 +35,8 @@ static  x_im_export_syms_t  im_export_syms =
 	ml_get_char_encoding_name ,
 	ml_get_char_encoding ,
 	ml_is_msb_set ,
-	ml_iscii_keymap_new ,
-	ml_iscii_keymap_delete ,
+	ml_isciikey_state_new ,
+	ml_isciikey_state_delete ,
 	ml_convert_ascii_to_iscii ,
 	ml_parser_new ,
 	ml_conv_new ,
