@@ -866,6 +866,8 @@ window_realized(
 
 	x_window_set_use_xft( &cand_screen->window ,
 		x_get_type_engine( cand_screen->font_man) == TYPE_XFT) ;
+	x_window_set_use_cairo( &cand_screen->window ,
+		x_get_type_engine( cand_screen->font_man) == TYPE_CAIRO) ;
 
 	x_window_set_fg_color( win , x_get_xcolor( cand_screen->color_man , ML_FG_COLOR)) ;
 	x_window_set_bg_color( win , x_get_xcolor( cand_screen->color_man , ML_BG_COLOR)) ;
