@@ -9,12 +9,6 @@
 #include  <kiklib/kik_str.h>	/* kik_str_to_uint */
 
 
-/* --- static variables --- */
-
-static char *  true = "true" ;
-static char *  false = "false" ;
-
-
 /* --- global functions --- */
 
 int
@@ -63,13 +57,14 @@ x_prepare_for_main_config(
 #ifndef  USE_WIN32GUI
 	kik_conf_add_opt( conf , '&' , "borderless" , 1 , "borderless" ,
 		"override redirect [false]") ;
-#if  defined(USE_TYPE_XCORE)
 	kik_conf_add_opt( conf , '*' , "type" , 0 , "type_engine" ,
+	#if  defined(USE_TYPE_XCORE)
 		"type engine [xcore]") ;
-#elif  defined(USE_TYPE_XFT)
-	kik_conf_add_opt( conf , '*' , "type" , 0 , "type_engine" ,
+	#elif  defined(USE_TYPE_XFT)
 		"type engine [xft]") ;
-#endif
+	#elif  defined(USE_TYPE_CAIRO)
+		"type engine [cairo]") ;
+	#endif
 #endif	/* USE_WIN32GUI */
 
 	kik_conf_add_opt( conf , '1' , "wscr" , 0 , "screen_width_ratio" ,
@@ -92,15 +87,15 @@ x_prepare_for_main_config(
 		"cursor foreground color") ;
 	kik_conf_add_opt( conf , '0' , "crbg" , 0 , "cursor_bg_color" ,
 		"cursor background color") ;
-#ifdef  USE_TYPE_XFT
+#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	kik_conf_add_opt( conf , 'A' , "aa" , 1 , "use_anti_alias" , 
-		"forcibly use anti alias font by using Xft") ;
+		"forcibly use anti alias font by using Xft or cairo") ;
 #endif
 	kik_conf_add_opt( conf , 'B' , "sbbg" , 0 , "sb_bg_color" , 
 		"scrollbar background color") ;
 #ifdef  USE_IND
-	kik_conf_add_opt( conf , 'C' , "iscii" , 0 , "iscii_lang" , 
-		"language to be used in ISCII encoding") ;
+	kik_conf_add_opt( conf , 'C' , "ind" , 1 , "use_ind" , 
+		"use indic (ligature text) [true(if use_bidi is false)]") ;
 #endif
 #ifdef  USE_FRIBIDI
 	kik_conf_add_opt( conf , 'D' , "bi" , 1 , "use_bidi" , 
@@ -244,8 +239,10 @@ x_main_config_init(
 {
 	char *  value ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
-	
+
+#ifndef  USE_WIN32GUI
 	if( ( value = kik_conf_get_value( conf , "display")) == NULL)
+#endif
 	{
 		value = "" ;
 	}
@@ -298,10 +295,12 @@ x_main_config_init(
 
 	main_config->icon_name = NULL ;
 
+#ifndef  USE_WIN32GUI
 	if( ( value = kik_conf_get_value( conf , "icon_name")))
 	{
 		main_config->icon_name = strdup( value) ;
 	}
+#endif
 
 	/* Not changeable. */
 	main_config->conf_menu_path_1 = NULL ;
@@ -338,7 +337,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "use_combining")))
 	{
-		if( strcmp( value , false) == 0)
+		if( strcmp( value , "false") == 0)
 		{
 			main_config->use_char_combining = 0 ;
 		}
@@ -347,7 +346,7 @@ x_main_config_init(
 	main_config->use_dynamic_comb = 0 ;
 	if( ( value = kik_conf_get_value( conf , "use_dynamic_comb")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->use_dynamic_comb = 1 ;
 		}
@@ -356,7 +355,7 @@ x_main_config_init(
 	main_config->logging_vt_seq = 0 ;
 	if( ( value = kik_conf_get_value( conf , "logging_vt_seq")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->logging_vt_seq = 1 ;
 		}
@@ -366,7 +365,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_variable_column_width")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->font_present |= FONT_VAR_WIDTH ;
 		}
@@ -388,44 +387,36 @@ x_main_config_init(
 		}
 	}
 
-#ifdef  USE_TYPE_XCORE
+#if  defined(USE_TYPE_XCORE)
 	main_config->type_engine = TYPE_XCORE ;
-#else
+#elif  defined(USE_TYPE_XFT)
 	main_config->type_engine = TYPE_XFT ;
+#else
+	main_config->type_engine = TYPE_CAIRO ;
 #endif
 
 	if( ( value = kik_conf_get_value( conf , "type_engine")))
 	{
-	#ifdef  USE_TYPE_XFT
-		if( strcmp( value , "xft") == 0)
-		{
-			main_config->type_engine = TYPE_XFT ;
-		}
-		else
-	#endif
-
-	#ifdef  USE_TYPE_XCORE
-		if( strcmp( value , "xcore") == 0)
-		{
-			main_config->type_engine = TYPE_XCORE ;
-		}
-		else
-	#endif
-		{
-			kik_msg_printf( "%s is unsupported type engine.\n" , value) ;
-		}
+		main_config->type_engine = x_get_type_engine_by_name( value) ;
 	}
 
-#ifdef  USE_TYPE_XFT
+#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( ( value = kik_conf_get_value( conf , "use_anti_alias")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->font_present |= FONT_AA ;
-			/* forcibly use xft */
-			main_config->type_engine = TYPE_XFT ;
+			if( main_config->type_engine == TYPE_XCORE)
+			{
+				/* forcibly use xft or cairo */
+			#ifdef  USE_TYPE_XFT
+				main_config->type_engine = TYPE_XFT ;
+			#else
+				main_config->type_engine = TYPE_CAIRO ;
+			#endif
+			}
 		}
-		else if( strcmp( value , false) == 0)
+		else if( strcmp( value , "false") == 0)
 		{
 			main_config->font_present |= FONT_NOAA ;
 		}
@@ -585,7 +576,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_multi_column_char")))
 	{
-		if( strcmp( value , false) == 0)
+		if( strcmp( value , "false") == 0)
 		{
 			main_config->use_multi_col_char = 0 ;
 		}
@@ -652,7 +643,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "use_login_shell")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->use_login_shell = 1 ;
 		}
@@ -662,7 +653,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "big5_buggy")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->big5_buggy = 1 ;
 		}
@@ -672,7 +663,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_scrollbar")))
 	{
-		if( strcmp( value , false) == 0)
+		if( strcmp( value , "false") == 0)
 		{
 			main_config->use_scrollbar = 0 ;
 		}
@@ -680,7 +671,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "scrollbar_mode")))
 	{
-		main_config->sb_mode = x_get_sb_mode( value) ;
+		main_config->sb_mode = x_get_sb_mode_by_name( value) ;
 	}
 	else
 	{
@@ -691,7 +682,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "iso88591_font_for_usascii")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->iso88591_font_for_usascii = 1 ;
 		}
@@ -701,7 +692,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "not_use_unicode_font")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->unicode_policy = NOT_USE_UNICODE_FONT ;
 		}
@@ -709,7 +700,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "only_use_unicode_font")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			if( main_config->unicode_policy == NOT_USE_UNICODE_FONT)
 			{
@@ -729,7 +720,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_unicode_property")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			if( main_config->unicode_policy == ONLY_USE_UNICODE_FONT)
 			{
@@ -748,7 +739,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "receive_string_via_ucs")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->receive_string_via_ucs = 1 ;
 		}
@@ -785,7 +776,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_transbg")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->use_transbg = 1 ;
 		}
@@ -904,11 +895,12 @@ x_main_config_init(
 		main_config->encoding = ML_ISO8859_1 ;
 	}
 
+#ifdef  USE_FRIBIDI
 	main_config->use_bidi = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_bidi")))
 	{
-		if( strcmp( value , false) == 0)
+		if( strcmp( value , "false") == 0)
 		{
 			main_config->use_bidi = 0 ;
 		}
@@ -920,6 +912,21 @@ x_main_config_init(
 	{
 		main_config->bidi_mode = ml_get_bidi_mode( value) ;
 	}
+#else
+	main_config->use_bidi = 0 ;
+	main_config->bidi_mode = BIDI_NORMAL_MODE ;
+#endif
+
+#ifdef  USE_IND
+	main_config->use_ind = 1 ;
+
+	if( ( value = kik_conf_get_value( conf , "use_ind")))
+	{
+		if( strcmp( value , "false") == 0)
+		{
+			main_config->use_ind = 0 ;
+		}
+	}
 
 	/* If value is "none" or not is also checked in x_screen.c */
 	if( ( value = kik_conf_get_value( conf , "mod_meta_key")) &&
@@ -931,10 +938,13 @@ x_main_config_init(
 	{
 		main_config->mod_meta_key = NULL ;
 	}
+#else
+	main_config->use_ind = 0 ;
+#endif
 	
 	if( ( value = kik_conf_get_value( conf , "mod_meta_mode")))
 	{
-		main_config->mod_meta_mode = x_get_mod_meta_mode( value) ;
+		main_config->mod_meta_mode = x_get_mod_meta_mode_by_name( value) ;
 	}
 	else
 	{
@@ -943,7 +953,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "bel_mode")))
 	{
-		main_config->bel_mode = x_get_bel_mode( value) ;
+		main_config->bel_mode = x_get_bel_mode_by_name( value) ;
 	}
 	else
 	{
@@ -954,29 +964,17 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_vertical_cursor")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->use_vertical_cursor = 1 ;
 		}
 	}
 	
-	main_config->iscii_lang_type = ISCIILANG_MALAYALAM ;
-
-	if( ( value = kik_conf_get_value( conf , "iscii_lang")))
-	{
-		mkf_iscii_lang_t  type ;
-		
-		if( ( type = ml_iscii_get_lang( value)) != ISCIILANG_UNKNOWN)
-		{
-			main_config->iscii_lang_type = type ;
-		}
-	}
-
 	main_config->use_extended_scroll_shortcut = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "use_extended_scroll_shortcut")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->use_extended_scroll_shortcut = 1 ;
 		}
@@ -986,7 +984,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "borderless")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->borderless = 1 ;
 		}
@@ -996,7 +994,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "static_backscroll_mode")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->bs_mode = BSM_STATIC ;
 		}
@@ -1143,7 +1141,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "skip_dialog")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			main_config->skip_dialog = 1 ;
 		}
@@ -1166,7 +1164,7 @@ x_main_config_init(
 	}
 #endif
 
-	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , true) == 0)
+	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , "true") == 0)
 	{
 		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))) == NULL)
 		{
