@@ -12,29 +12,39 @@
 #include  "x_window.h"
 
 
+#define  x_picture_modifier_is_normal(pic_mod) (x_picture_modifiers_equal((pic_mod), NULL))
+
+
 typedef struct x_picture_modifier
 {
-	u_int16_t  brightness ;		/* 0 - 65536 */
-	u_int16_t  contrast ;		/* 0 - 65536 */
-	u_int16_t  gamma ;		/* 0 - 65536 */
+	u_int16_t  brightness ;		/* 0 - 65535 */
+	u_int16_t  contrast ;		/* 0 - 65535 */
+	u_int16_t  gamma ;		/* 0 - 65535 */
+
+	u_int8_t  alpha ;		/* 0 - 255 */
+	u_int8_t  blend_red ;
+	u_int8_t  blend_green ;
+	u_int8_t  blend_blue ;
 
-	u_long  blend_color ;		/* Not used */
-	u_int8_t  alpha ;		/* Used only in win32. */
-	
 } x_picture_modifier_t ;
 
 typedef struct x_bg_picture
 {
-	x_window_t *  win ;
+	Display *  display ;
 	x_picture_modifier_t *  mod ;
+	char *  file_path ;
+	u_int  width ;
+	u_int  height ;
 	
 	Pixmap  pixmap ;
 
+	u_int  ref_count ;
+
 } x_bg_picture_t ;
 
 typedef struct x_icon_picture
 {
-	Display *  display ;
+	x_display_t *  disp ;
 	char *  file_path ;
 	
 	Pixmap  pixmap ;
@@ -50,19 +60,16 @@ int  x_picture_display_opened( Display *  display) ;
 
 int  x_picture_display_closed( Display *  display) ;
 
-int  x_picture_modifier_is_normal( x_picture_modifier_t *  pic_mod) ;
+int  x_picture_modifiers_equal( x_picture_modifier_t *  a , x_picture_modifier_t *  b) ;
 
 int  x_root_pixmap_available( Display *  display) ;
 
-int  x_bg_picture_init( x_bg_picture_t *  pic , x_window_t *  win , x_picture_modifier_t *  mod) ;
+x_bg_picture_t *  x_acquire_bg_picture( x_window_t *  win , x_picture_modifier_t *  mod ,
+			char *  file_path) ;
 
-int  x_bg_picture_final( x_bg_picture_t *  pic) ;
+int  x_release_bg_picture( x_bg_picture_t *  pic) ;
 
-int  x_bg_picture_load_file( x_bg_picture_t *  pic , char *  file_path) ;
-
-int  x_bg_picture_get_transparency( x_bg_picture_t *  pic) ;
-
-x_icon_picture_t *  x_acquire_icon_picture( Display *  display , char *  file_path) ;
+x_icon_picture_t *  x_acquire_icon_picture( x_display_t *  disp , char *  file_path) ;
 
 int  x_release_icon_picture( x_icon_picture_t *  pic) ;
 
