@@ -1,5 +1,5 @@
 /*
- *	$Id: x_shortcut.h,v 1.9 2004/12/13 09:31:55 h_minami Exp $
+ *	$Id$
  */
 
 /*
@@ -10,7 +10,7 @@
 #define  __X_SHORTCUT_H__
 
 
-#include  <X11/Xlib.h>
+#include  "x.h"
 #include  <kiklib/kik_types.h>
 
 
@@ -62,8 +62,6 @@ int  x_shortcut_init( x_shortcut_t *  shortcut) ;
 
 int  x_shortcut_final( x_shortcut_t *  shortcut) ;
 
-int  x_read_shortcut_config( x_shortcut_t *  shortcut , char *  filename) ;
-
 int  x_shortcut_match( x_shortcut_t *  shortcut , x_key_func_t  func , KeySym  sym , u_int  state) ;
 
 char *  x_shortcut_str( x_shortcut_t *  shortcut , KeySym  sym , u_int  state) ;
