@@ -192,7 +192,7 @@ static int
 get_font_size_range(
 	u_int *  min ,
 	u_int *  max ,
-	char *  str
+	const char *  str
 	)
 {
 	char *  str_p ;
@@ -245,11 +245,11 @@ create_term_intern(void)
 			main_config.encoding , main_config.is_auto_encoding , 
 			main_config.unicode_policy , main_config.col_size_of_width_a ,
 			main_config.use_char_combining , main_config.use_multi_col_char ,
-			main_config.use_bidi , main_config.bidi_mode ,
+			main_config.use_bidi , main_config.bidi_mode , main_config.use_ind ,
 			x_termcap_get_bool_field(
 				x_termcap_get_entry( &termcap , main_config.term_type) , ML_BCE) ,
 			main_config.use_dynamic_comb , main_config.bs_mode ,
-			main_config.vertical_mode , main_config.iscii_lang_type)) == NULL)
+			main_config.vertical_mode)) == NULL)
 	{
 		return  NULL ;
 	}
@@ -1668,7 +1668,6 @@ x_term_manager_init(
 	u_int  min_font_size ;
 	u_int  max_font_size ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
-	char *  true = "true" ;
 
 	if( ! x_color_config_init( &color_config))
 	{
@@ -1762,6 +1761,10 @@ x_term_manager_init(
 		) ;
 	kik_conf_add_opt( conf , '\0' , "clip" , 1 , "use_clipboard" ,
 		"use CLIPBOARD (not only PRIMARY) selection [false]") ;
+#ifdef  USE_IM_CURSOR_COLOR
+	kik_conf_add_opt( conf , '\0' , "imcolor" , 0 , "im_cursor_color" ,
+		"cursor color when input method is activated.") ;
+#endif
 
 	if( ! kik_conf_parse_args( conf , &argc , &argv))
 	{
@@ -1858,7 +1861,7 @@ x_term_manager_init(
 
 	if( ( value = kik_conf_get_value( conf , "compose_dec_special_font")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			x_compose_dec_special_font() ;
 		}
@@ -1866,9 +1869,9 @@ x_term_manager_init(
 
 #ifdef  USE_TYPE_XFT
 	if( ( value = kik_conf_get_value( conf , "use_cp932_ucs_for_xft")) == NULL ||
-		strcmp( value , true) == 0)
+		strcmp( value , "true") == 0)
 	{
-		ml_use_cp932_ucs_for_xft() ;
+		x_use_cp932_ucs_for_xft() ;
 	}
 #endif
 
@@ -1971,12 +1974,22 @@ x_term_manager_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_clipboard")))
 	{
-		if( strcmp( value , true) == 0)
+		if( strcmp( value , "true") == 0)
 		{
 			x_set_use_clipboard_selection( 1) ;
 		}
 	}
 
+#ifdef  USE_IM_CURSOR_COLOR
+	if( ( value = kik_conf_get_value( conf , "im_cursor_color")))
+	{
+		if( *value)
+		{
+			x_set_im_cursor_color( value) ;
+		}
+	}
+#endif
+
 
 	if( ( version = kik_conf_get_version( conf)) == NULL)
 	{
