@@ -16,6 +16,11 @@
 #include  <X11/Xft/Xft.h>
 #endif
 
+#ifdef  USE_TYPE_CAIRO
+#include  <cairo/cairo.h>
+#include  <fontconfig/fontconfig.h>
+#endif
+
 #include  <kiklib/kik_types.h>
 #include  <mkf/mkf_parser.h>
 #include  <ml_char.h>
@@ -74,7 +79,12 @@ typedef struct  x_window
 #ifdef  USE_TYPE_XFT
 	XftDraw *  xft_draw ;
 #else
-	void *  xft_draw ; /* dummy to keep the size of x_window_t */
+	void *  xft_draw ;		/* dummy to keep the size of x_window_t */
+#endif
+#ifdef  USE_TYPE_CAIRO
+	cairo_t *  cairo_draw ;
+#else
+	void *  cairo_draw ;		/* dummy to keep the size of x_window_t */
 #endif
 
 	x_color_t  fg_color ;
@@ -193,6 +203,8 @@ int  x_window_final( x_window_t *  win) ;
 
 int  x_window_set_use_xft( x_window_t *  win , int  use_xft) ;
 
+int  x_window_set_use_cairo( x_window_t *  win , int  use_cairo) ;
+
 int  x_window_init_event_mask( x_window_t *  win , long  event_mask) ;
 
 int  x_window_add_event_mask( x_window_t *  win , long  event_mask) ;
@@ -323,12 +335,12 @@ int  x_window_draw_image_string16( x_window_t *  win , x_font_t *  font ,
 	XChar2b *  str , u_int  len) ;
 #endif
 
-#ifdef  USE_TYPE_XFT
-int  x_window_xft_draw_string8( x_window_t *  win , x_font_t *  font ,
+#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+int  x_window_ft_draw_string8( x_window_t *  win , x_font_t *  font ,
 	x_color_t *  fg_color , int  x , int  y , u_char *  str , size_t  len) ;
 
-int  x_window_xft_draw_string32( x_window_t *  win , x_font_t *  font ,
-	x_color_t *  fg_color , int  x , int  y , XftChar32 *  str , u_int  len) ;
+int  x_window_ft_draw_string32( x_window_t *  win , x_font_t *  font ,
+	x_color_t *  fg_color , int  x , int  y , FcChar32 *  str , u_int  len) ;
 #endif
 
 int  x_window_draw_rect_frame( x_window_t *  win , int  x1 , int  y1 , int  x2 , int  y2) ;
